import os
from typing import Tuple
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.backends import default_backend


class PemKeyCreator:
    PUBLIC_EXPONENT = 65537

    PRIVATE_KEY_STR = 'private.key'
    PUBLIC_KEY_STR = 'public.key'

    @staticmethod
    def _save_key(key, filepath: str) -> None:
        with open(filepath, 'wb') as fp:
            fp.write(key)

    @classmethod
    def create_key_pair(cls, path: str='') -> Tuple[str, str]:
        key = rsa.generate_private_key(
            backend=default_backend(),
            public_exponent=cls.PUBLIC_EXPONENT,
            key_size=2048
        )

        # get private key (RSA PRIVATE KEY)
        private_key = key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()
        )

        # get public key (RSA PUBLIC KEY)
        public_key = key.public_key().public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )

        private_key_path = os.path.join(path, cls.PRIVATE_KEY_STR)
        cls._save_key(private_key, private_key_path)
        public_key_path = os.path.join(path, cls.PUBLIC_KEY_STR)
        cls._save_key(public_key, public_key_path)

        return private_key_path, public_key_path



import os
from typing import Callable
from cryptography.hazmat.primitives.serialization import load_pem_private_key, load_pem_public_key
from cryptography.hazmat.backends import default_backend


class PemKeyLoader:

    @staticmethod
    def _load_key(filepath: str, load_function: Callable, **kwargs):
        key = None
        if os.path.isfile(filepath):
            with open(filepath, 'rb') as fp:
                key = load_function(fp.read(), **kwargs)

        return key

    @classmethod
    def load_private_key(cls, filepath: str):
        return cls._load_key(filepath, load_pem_private_key, password=None, backend=default_backend())

    @classmethod
    def load_public_key(cls, filepath: str):
        return cls._load_key(filepath, load_pem_public_key, backend=default_backend())



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="j_crypto",
    version="0.0.1",
    author="Acciaioli Valverde",
    author_email="acci.valverde@gmail.com",
    description="Helper fucntions to deal with RSA keys",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/foothub/crypto-utils",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
    ],
    scripts=['bin/j-crypto-create-pair'],
    install_requires=['pyjwt', 'cryptography'],
)    
